use std::{env::consts::DLL_EXTENSION, fs::create_dir_all};

use clap::Parser;
use git2::Repository;

mod cc;
mod languages;
mod settings;

use languages::{InstallInfo, get_install_info};
use settings::{Settings, get_settings};

#[derive(Debug, Parser)]
#[command(version, about, long_about = None)]
struct Cli {
    language: String,
}

fn main() -> anyhow::Result<()> {
    let Cli { language } = Cli::parse();

    let Settings { parser_dir } = get_settings()?;

    let src_dir = tempfile::tempdir()?;

    create_dir_all(&parser_dir)?;

    // have to use a canonical path bc compiler runs elsewhere
    let parser_dir = parser_dir.canonicalize()?;

    let InstallInfo { url, files } =
        get_install_info(&language).ok_or(anyhow::anyhow!("unknown language `{language}`"))?;

    let repo_path = src_dir.as_ref().join(&language);
    Repository::clone(url, &repo_path)?;

    let output_path = parser_dir.join(language).with_extension(DLL_EXTENSION);

    let mut build = cc::Build::new();
    build.dir(&repo_path).input_files(files);
    build.compile(&output_path)?;

    // close explicitly to prevent premature drop
    src_dir.close()?;

    Ok(())
}
